'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.interval = undefined;

var _generator = require('../generator');

var _options = require('../options');

var _utility = require('../utility');

var doChunkSync = function doChunkSync(chunkIterator, indexConsumer) {
    var next = chunkIterator.next();
    var value = undefined;
    while (!next.done) {
        value = next.value;
        if (typeof value !== 'number') {
            return value;
        }
        try {
            indexConsumer(value);
        } catch (error) {
            throw { error: error, index: value };
        }
        next = chunkIterator.next();
    }
    return null;
};
var interval = exports.interval = function interval(indexConsumer, start, final) {
    var options = arguments.length <= 3 || arguments[3] === undefined ? _options.DEFAULT_OPTIONS : arguments[3];

    (0, _utility.assertFn)(indexConsumer);
    (0, _utility.assertNumber)(start);
    (0, _utility.assertNumber)(final);
    if (start >= final) {
        throw new Error('Expected start ' + start + ' to be less than final ' + final);
    }
    var chOptions = (0, _options.parseOptions)(options);
    var chunkIterator = (0, _generator.generator)(start, final, chOptions);
    var boundConsumer = function boundConsumer(index) {
        // FIXME - this (and similar upstream invocations) are untyped!
        //
        //   https://github.com/Microsoft/TypeScript/issues/212
        indexConsumer.call(chOptions.scope, index);
    };
    var nextChunkExecutor = function nextChunkExecutor(complete) {
        var nextPause = doChunkSync(chunkIterator, boundConsumer);
        if (nextPause) {
            return nextPause.resume(function () {
                return nextChunkExecutor(complete);
            });
        }
        complete();
    };
    return new Promise(nextChunkExecutor);
};