'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.parseOptions = exports.DEFAULT_OPTIONS = undefined;

var _utility = require('./utility');

var DEFAULT_OPTIONS = exports.DEFAULT_OPTIONS = Object.freeze({
    chunk: 10,
    delay: 50,
    scope: null
});
var CHUNK = { primary: 'chunk', aliases: ['chunksize'] };
var SCOPE = { primary: 'scope', aliases: [] };
var DELAY = { primary: 'delay', aliases: ['yield', 'yieldtime', 'delaytime']
};
var checkOption = function checkOption(optionKey, value) {
    switch (optionKey) {
        case CHUNK:
            if (!(0, _utility.isNumber)(value) || value <= 0) {
                throw new TypeError('\'' + optionKey.primary + '\' should be a positive number');
            }
            break;
        case DELAY:
            if (!(0, _utility.isNumber)(value) || value < 0) {
                throw new TypeError('\'' + optionKey.primary + '\' should be a non-negative number');
            }
            break;
        case SCOPE:
            if (value === undefined || (0, _utility.isBoolean)(value) || (0, _utility.isNumber)(value)) {
                throw new TypeError('\'' + optionKey.primary + '\' should not be undefined, a boolean, or a number');
            }
            break;
        default:
            throw new TypeError('Unknown option key ' + optionKey);
    }
};
var mapsToOption = function mapsToOption(optionKey, key) {
    return optionKey.primary === key || optionKey.aliases.indexOf(key) > -1;
};
var setOption = function setOption(object, optionKey, value) {
    checkOption(optionKey, value);
    object[optionKey.primary] = value;
};
var parseOptions = exports.parseOptions = function parseOptions(options) {
    (0, _utility.assertIsPlainObject)(options);
    var parsed = (0, _utility.extend)({}, DEFAULT_OPTIONS);
    (0, _utility.forOwn)(options, function (value, key) {
        if (mapsToOption(CHUNK, key)) {
            setOption(parsed, CHUNK, value);
        } else if (mapsToOption(DELAY, key)) {
            setOption(parsed, DELAY, value);
        } else if (mapsToOption(SCOPE, key)) {
            setOption(parsed, SCOPE, value);
        }
    });
    return Object.freeze(parsed);
};