'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

Object.defineProperty(exports, "__esModule", {
    value: true
});
var forOwn = exports.forOwn = function forOwn(iteratee, iterator) {
    for (var key in iteratee) {
        if (iteratee.hasOwnProperty(key)) {
            iterator(iteratee[key], key);
        }
    }
};
var isBoolean = exports.isBoolean = function isBoolean(thing) {
    return typeof thing === 'boolean';
};
var isNumber = exports.isNumber = function isNumber(thing) {
    return typeof thing === 'number';
};
var defaults = exports.defaults = function defaults(defaultsObj, overrideObj) {
    forOwn(overrideObj, function (value, key) {
        if (!defaultsObj.hasOwnProperty(key)) {
            defaultsObj[key] = overrideObj[key];
        }
    });
    return defaultsObj;
};
var extend = exports.extend = function extend(targetObj, sourceObj) {
    forOwn(sourceObj, function (value, key) {
        targetObj[key] = sourceObj[key];
    });
    return targetObj;
};
var assertNonemptyArray = exports.assertNonemptyArray = function assertNonemptyArray(thing) {
    if (!Array.isArray(thing) || !thing.length) {
        throw new TypeError('Expected non-empty array, got ' + (typeof thing === 'undefined' ? 'undefined' : _typeof(thing)));
    }
};
var isFunction = function isFunction(thing) {
    return typeof thing === 'function';
};
var assertIsPlainObject = exports.assertIsPlainObject = function assertIsPlainObject(thing) {
    var isPlainObject = function isPlainObject() {
        // slightly modified from:
        //
        //    https://github.com/lodash/lodash/blob/master/lodash.js#L9976
        var isObjectLike = function isObjectLike() {
            return !!thing && (typeof thing === 'undefined' ? 'undefined' : _typeof(thing)) === 'object';
        };
        var objectProto = Object.prototype;
        if (!isObjectLike() || objectProto.toString.call(thing) !== '[object Object]') {
            return false;
        }
        var proto = objectProto;
        if (isFunction(thing.constructor)) {
            proto = Object.getPrototypeOf(thing);
        }
        if (!proto) {
            return true;
        }
        var fnToString = Function.prototype.toString;
        var ctor = proto.constructor;
        return isFunction(ctor) && ctor instanceof ctor && fnToString.call(ctor) === fnToString.call(Object);
    };
    if (!isPlainObject()) {
        throw new TypeError('Expected plain javascript object, got ' + (typeof thing === 'undefined' ? 'undefined' : _typeof(thing)));
    }
};
var assertNumber = exports.assertNumber = function assertNumber(thing) {
    if (!isNumber(thing)) {
        throw new TypeError('Expected number, got ' + (typeof thing === 'undefined' ? 'undefined' : _typeof(thing)));
    }
};
var assertFn = exports.assertFn = function assertFn(thing) {
    if (!isFunction(thing)) {
        throw new TypeError('Expected function, got ' + (typeof thing === 'undefined' ? 'undefined' : _typeof(thing)));
    }
};
var slice = [].slice;
var compose = exports.compose = function compose(f, g) {
    assertFn(f);
    assertFn(g);
    return function () {
        return f(g.apply(this, slice.call(arguments)));
    };
};